@@ -17,12 +17,7 @@ from datetime import datetime
 class InvalidParameterException(Exception):
     """Exception for invalid parameter args provided by the user."""
 
-    def __init__(self, message: str):
-        """Create new instance
-
-        :param str message: string describing the nature of the error
-        """
-        super().__init__(message)
+    pass
 
 
 class QueryHandler:
@@ -86,7 +81,7 @@ class QueryHandler:
         :param Dict[str, Dict] response: in-progress response object to return
             to client
         :param Dict item: Item retrieved from DynamoDB
-        :param MatchTy pe match_type: type of query match
+        :param MatchType match_type: match type for query
         :return: Tuple containing updated response object, and string
             containing name of the source of the match
         """
@@ -99,6 +94,7 @@ class QueryHandler:
                         int(loc['interval']['start']['value'])
                     loc['interval']['end']['value'] = \
                         int(loc['interval']['end']['value'])
+        item["match_type"] = match_type
         gene = Gene(**item)
         src_name = item['src_name']
 
@@ -107,47 +103,36 @@ class QueryHandler:
             pass
         elif matches[src_name] is None:
             matches[src_name] = {
-                'match_type': match_type,
                 'records': [gene],
                 'source_meta_': self.fetch_meta(src_name)
             }
-        elif matches[src_name]['match_type'].value == match_type.value:
+        else:
             matches[src_name]['records'].append(gene)
 
         return response, src_name
 
-    def fetch_records(self,
-                      response: Dict[str, Dict],
-                      concept_ids: List[str],
-                      match_type: MatchType) -> (Dict, Set):
-        """Return matched Gene records as a structured response for a given
-        collection of concept IDs.
+    def fetch_record(self, response: Dict[str, Dict], concept_id: str,
+                     match_type: MatchType) -> None:
+        """Add fetched record to response
 
         :param Dict[str, Dict] response: in-progress response object to return
             to client.
-        :param List[str] concept_ids: List of concept IDs to build from.
+        :param str concept_id: Concept id to fetch record for.
             Should be all lower-case.
-        :param MatchType match_type: record should be assigned this type of
-            match.
-        :return: response Dict with records filled in via provided concept
-            IDs, and Set of source names of matched records
+        :param MatchType match_type: match type for record
         """
-        matched_sources = set()
-        for concept_id in concept_ids:
-            try:
-                pk = f'{concept_id.lower()}##identity'
-                filter_exp = Key('label_and_type').eq(pk)
-                result = self.db.genes.query(KeyConditionExpression=filter_exp)
-                match = result['Items'][0]
-                (response, src) = self.add_record(response, match, match_type)
-                matched_sources.add(src)
-            except ClientError as e:
-                logger.error(e.response['Error']['Message'])
+        try:
+            pk = f'{concept_id}##identity'
+            filter_exp = Key('label_and_type').eq(pk)
+            result = self.db.genes.query(KeyConditionExpression=filter_exp)
+            match = result['Items'][0]
+            self.add_record(response, match, match_type)
+        except ClientError as e:
+            logger.error(e.response['Error']['Message'])
 
-        return response, matched_sources
-
-    def fill_no_matches(self, resp: Dict) -> Dict:
-        """Fill all empty source_matches slots with NO_MATCH results.
+    def post_process_resp(self, resp: Dict) -> Dict:
+        """Fill all empty source_matches slots with NO_MATCH results and
+        sort source records by descending `match_type`.
 
         :param Dict resp: incoming response object
         :return: response object with empty source slots filled with
@@ -160,79 +145,13 @@ class QueryHandler:
                     'records': [],
                     'source_meta_': self.fetch_meta(src_name)
                 }
+            else:
+                records = resp['source_matches'][src_name]['records']
+                if len(records) > 1:
+                    records = sorted(
+                        records, key=lambda k: k.match_type, reverse=True)
         return resp
 
-    def check_concept_id(self,
-                         query: str,
-                         resp: Dict,
-                         sources: Set[str]) -> (Dict, Set):
-        """Check query for concept ID match. Should only find 0 or 1 matches,
-        but stores them as a collection to be safe.
-
-        :param str query: search string
-        :param Dict resp: in-progress response object to return to client
-        :param Set[str] sources: remaining unmatched sources
-        :return: Tuple with updated resp object and updated set of unmatched
-            sources
-        """
-        concept_id_items = []
-        if [p for p in PREFIX_LOOKUP.keys() if query.startswith(p)]:
-            pk = f'{query}##identity'
-            filter_exp = Key('label_and_type').eq(pk)
-            try:
-                result = self.db.genes.query(KeyConditionExpression=filter_exp)
-                if len(result['Items']) > 0:
-                    concept_id_items += result['Items']
-            except ClientError as e:
-                logger.error(e.response['Error']['Message'])
-        for prefix in [p for p in NAMESPACE_LOOKUP.keys() if
-                       query.startswith(p)]:
-            pk = f'{NAMESPACE_LOOKUP[prefix].lower()}:{query}##identity'
-            filter_exp = Key('label_and_type').eq(pk)
-            try:
-                result = self.db.genes.query(
-                    KeyConditionExpression=filter_exp
-                )
-                if len(result['Items']) > 0:
-                    concept_id_items += result['Items']
-            except ClientError as e:
-                logger.error(e.response['Error']['Message'])
-
-        for item in concept_id_items:
-            (resp, src_name) = self.add_record(resp, item,
-                                               MatchType.CONCEPT_ID)
-            sources = sources - {src_name}
-        return resp, sources
-
-    def check_match_type(self,
-                         query: str,
-                         resp: Dict,
-                         sources: Set[str],
-                         match: str) -> (Dict, Set):
-        """Check query for selected match type.
-
-        :param str query: search string
-        :param Dict resp: in-progress response object to return to client
-        :param Set[str] sources: remaining unmatched sources
-        :param str match: Match type name
-        :return: Tuple with updated resp object and updated set of unmatched
-                 sources
-        """
-        filter_exp = Key('label_and_type').eq(f'{query}##{match}')
-        try:
-            db_response = self.db.genes.query(
-                KeyConditionExpression=filter_exp
-            )
-            if 'Items' in db_response.keys():
-                concept_ids = [i['concept_id'] for i in db_response['Items']]
-                (resp, matched_srcs) = self.fetch_records(
-                    resp, concept_ids, MatchType[match.upper()]
-                )
-                sources = sources - matched_srcs
-        except ClientError as e:
-            logger.error(e.response['Error']['Message'])
-        return resp, sources
-
     def response_keyed(self, query: str, sources: Set[str]) -> Dict:
         """Return response as dict where key is source name and value
         is a list of records. Corresponds to `keyed=true` API parameter.
@@ -249,25 +168,48 @@ class QueryHandler:
             }
         }
         if query == '':
-            resp = self.fill_no_matches(resp)
-            return resp
+            return self.post_process_resp(resp)
         query_l = query.lower()
 
-        # check if concept ID match
-        (resp, sources) = self.check_concept_id(query_l, resp, sources)
-        if len(sources) == 0:
-            return resp
+        queries = list()
+        if [p for p in PREFIX_LOOKUP.keys() if query_l.startswith(p)]:
+            pk = f'{query_l}##identity'
+            queries.append(pk)
+
+        for prefix in [p for p in NAMESPACE_LOOKUP.keys() if
+                       query_l.startswith(p)]:
+            pk = f'{NAMESPACE_LOOKUP[prefix].lower()}:{query_l}##identity'
+            queries.append(pk)
 
         for match in ITEM_TYPES.values():
-            (resp, sources) = self.check_match_type(
-                query_l, resp, sources, match)
-            if len(sources) == 0:
-                return resp
+            pk = f'{query_l}##{match}'
+            queries.append(pk)
+
+        matched_concept_ids = list()
+        for q in queries:
+            try:
+                query_resp = self.db.genes.query(
+                    KeyConditionExpression=Key('label_and_type').eq(q)
+                )
+                for record in query_resp['Items']:
+                    concept_id = record['concept_id']
+                    if concept_id in matched_concept_ids:
+                        continue
+                    else:
+                        if record['item_type'] == "identity":
+                            self.add_record(resp, record, MatchType.CONCEPT_ID)
+                        else:
+                            self.fetch_record(
+                                resp, concept_id,
+                                MatchType[record['item_type'].upper()])
+                        matched_concept_ids.append(concept_id)
+
+            except ClientError as e:
+                logger.error(e.response['Error']['Message'])
+                continue
 
         # remaining sources get no match
-        resp = self.fill_no_matches(resp)
-
-        return resp
+        return self.post_process_resp(resp)
 
     def response_list(self, query: str, sources: Set[str]) -> Dict:
         """Return response as list, where the first key-value in each item
